@@ -1,6 +1,5 @@
 #! /usr/bin/python
 
-from __future__ import print_function
 import os, sys
 import shutil
 import tarfile
@@ -8,7 +7,7 @@ import unittest
 import tempfile
 
 
-from pkg_resources import parse_version, SetuptoolsLegacyVersion
+from pkg_resources import parse_version
 
 sys.path.insert(0, "src")
 import common
@@ -23,7 +22,8 @@ class ParseGitDescribe(unittest.TestCase):
         os.mkdir(self.fakegit)
 
     def test_pieces(self):
-        def pv(git_describe, do_error=False, expect_pieces=False):
+        def pv(git_describe, do_error=False,
+               expect_pieces=False, branch_name="master"):
             def fake_run_command(exes, args, cwd=None, hide_stderr=None):
                 if args[0] == "describe":
                     if do_error == "describe":
@@ -32,15 +32,24 @@ class ParseGitDescribe(unittest.TestCase):
                 if args[0] == "rev-parse":
                     if do_error == "rev-parse":
                         return None, 0
-                    return "longlong\n", 0
+                    if args[1] == "--abbrev-ref":
+                        return "%s\n" % branch_name, 0
+                    else:
+                        return "longlong\n", 0
                 if args[0] == "rev-list":
                     return "42\n", 0
                 if args[0] == "show":
+                    if do_error == "show":
+                        return "gpg: signature\n12345\n", 0
                     return "12345\n", 0
+                if args[0] == "branch":
+                    return "* (no branch)\n" \
+                           "  contained-branch-1\n" \
+                           "  contained-branch-2", 0
                 self.fail("git called in weird way: %s" % (args,))
             return from_vcs.git_pieces_from_vcs(
                 "v", self.fakeroot, verbose=False,
-                run_command=fake_run_command)
+                runner=fake_run_command)
         self.assertRaises(from_vcs.NotThisMethod,
                           pv, "ignored", do_error="describe")
         self.assertRaises(from_vcs.NotThisMethod,
@@ -50,37 +59,64 @@ class ParseGitDescribe(unittest.TestCase):
                           "distance": 42,
                           "long": "longlong",
                           "short": "longlon",
-                          "date": "12345"})
+                          "date": "12345",
+                          "branch": "master"})
+        self.assertEqual(pv("1f", do_error="show"),
+                         {"closest-tag": None, "dirty": False, "error": None,
+                          "distance": 42,
+                          "long": "longlong",
+                          "short": "longlon",
+                          "date": "12345",
+                          "branch": "master"})
         self.assertEqual(pv("1f-dirty"),
                          {"closest-tag": None, "dirty": True, "error": None,
                           "distance": 42,
                           "long": "longlong",
                           "short": "longlon",
-                          "date": "12345"})
+                          "date": "12345",
+                          "branch": "master"})
         self.assertEqual(pv("v1.0-0-g1f"),
                          {"closest-tag": "1.0", "dirty": False, "error": None,
                           "distance": 0,
                           "long": "longlong",
                           "short": "1f",
-                          "date": "12345"})
+                          "date": "12345",
+                          "branch": "master"})
         self.assertEqual(pv("v1.0-0-g1f-dirty"),
                          {"closest-tag": "1.0", "dirty": True, "error": None,
                           "distance": 0,
                           "long": "longlong",
                           "short": "1f",
-                          "date": "12345"})
+                          "date": "12345",
+                          "branch": "master"})
         self.assertEqual(pv("v1.0-1-g1f"),
                          {"closest-tag": "1.0", "dirty": False, "error": None,
                           "distance": 1,
                           "long": "longlong",
                           "short": "1f",
-                          "date": "12345"})
+                          "date": "12345",
+                          "branch": "master"})
         self.assertEqual(pv("v1.0-1-g1f-dirty"),
                          {"closest-tag": "1.0", "dirty": True, "error": None,
                           "distance": 1,
                           "long": "longlong",
                           "short": "1f",
-                          "date": "12345"})
+                          "date": "12345",
+                          "branch": "master"})
+        self.assertEqual(pv("v1.0-1-g1f-dirty", branch_name="feature-branch"),
+                         {"closest-tag": "1.0", "dirty": True, "error": None,
+                          "distance": 1,
+                          "long": "longlong",
+                          "short": "1f",
+                          "date": "12345",
+                          "branch": "feature-branch"})
+        self.assertEqual(pv("v1.0-1-g1f-dirty", branch_name="HEAD"),
+                         {"closest-tag": "1.0", "dirty": True, "error": None,
+                          "distance": 1,
+                          "long": "longlong",
+                          "short": "1f",
+                          "date": "12345",
+                          "branch": "contained-branch-1"})
 
     def tearDown(self):
         os.rmdir(self.fakegit)
@@ -88,9 +124,9 @@ class ParseGitDescribe(unittest.TestCase):
 
 
 class Keywords(unittest.TestCase):
-    def parse(self, refnames, full, prefix=""):
+    def parse(self, refnames, full, prefix="", date=None):
         return from_keywords.git_versions_from_keywords(
-            {"refnames": refnames, "full": full}, prefix, False)
+            {"refnames": refnames, "full": full, "date": date}, prefix, False)
 
     def test_parse(self):
         v = self.parse(" (HEAD, 2.0,master  , otherbranch ) ", " full ")
@@ -98,6 +134,7 @@ class Keywords(unittest.TestCase):
         self.assertEqual(v["full-revisionid"], "full")
         self.assertEqual(v["dirty"], False)
         self.assertEqual(v["error"], None)
+        self.assertEqual(v["date"], None)
 
     def test_prefer_short(self):
         v = self.parse(" (HEAD, 2.0rc1, 2.0, 2.0rc2) ", " full ")
@@ -105,6 +142,7 @@ class Keywords(unittest.TestCase):
         self.assertEqual(v["full-revisionid"], "full")
         self.assertEqual(v["dirty"], False)
         self.assertEqual(v["error"], None)
+        self.assertEqual(v["date"], None)
 
     def test_prefix(self):
         v = self.parse(" (HEAD, projectname-2.0) ", " full ", "projectname-")
@@ -112,6 +150,7 @@ class Keywords(unittest.TestCase):
         self.assertEqual(v["full-revisionid"], "full")
         self.assertEqual(v["dirty"], False)
         self.assertEqual(v["error"], None)
+        self.assertEqual(v["date"], None)
 
     def test_unexpanded(self):
         self.assertRaises(from_keywords.NotThisMethod,
@@ -123,6 +162,7 @@ class Keywords(unittest.TestCase):
         self.assertEqual(v["full-revisionid"], "full")
         self.assertEqual(v["dirty"], False)
         self.assertEqual(v["error"], "no suitable tags")
+        self.assertEqual(v["date"], None)
 
     def test_no_prefix(self):
         v = self.parse("(HEAD, master, 1.23)", "full", "missingprefix-")
@@ -130,14 +170,35 @@ class Keywords(unittest.TestCase):
         self.assertEqual(v["full-revisionid"], "full")
         self.assertEqual(v["dirty"], False)
         self.assertEqual(v["error"], "no suitable tags")
+        self.assertEqual(v["date"], None)
+
+    def test_date(self):
+        date = "2017-07-24 16:03:40 +0200"
+        result = "2017-07-24T16:03:40+0200"
+        v = self.parse(" (HEAD, 2.0,master  , otherbranch ) ", " full ",
+                       date=date)
+        self.assertEqual(v["date"], result)
+
+    def test_date_gpg(self):
+        date = """
+        gpg: Signature information
+        gpg: ...
+        2017-07-24 16:03:40 +0200"""
+        result = "2017-07-24T16:03:40+0200"
+        v = self.parse(" (HEAD, 2.0,master  , otherbranch ) ", " full ",
+                       date=date)
+        self.assertEqual(v["date"], result)
 
 expected_renders = """
 closest-tag: 1.0
 distance: 0
 dirty: False
+branch: feature
 pep440: 1.0
+pep440-branch: 1.0
 pep440-pre: 1.0
 pep440-post: 1.0
+pep440-post-branch: 1.0
 pep440-old: 1.0
 git-describe: 1.0
 git-describe-long: 1.0-0-g250b7ca
@@ -145,9 +206,12 @@ git-describe-long: 1.0-0-g250b7ca
 closest-tag: 1.0
 distance: 0
 dirty: True
+branch: master
 pep440: 1.0+0.g250b7ca.dirty
+pep440-branch: 1.0+0.g250b7ca.dirty
 pep440-pre: 1.0
 pep440-post: 1.0.post0.dev0+g250b7ca
+pep440-post-branch: 1.0.post0+g250b7ca.dirty
 pep440-old: 1.0.post0.dev0
 git-describe: 1.0-dirty
 git-describe-long: 1.0-0-g250b7ca-dirty
@@ -155,9 +219,12 @@ git-describe-long: 1.0-0-g250b7ca-dirty
 closest-tag: 1.0
 distance: 1
 dirty: False
+branch: master
 pep440: 1.0+1.g250b7ca
-pep440-pre: 1.0.post.dev1
+pep440-branch: 1.0+1.g250b7ca
+pep440-pre: 1.0.post0.dev1
 pep440-post: 1.0.post1+g250b7ca
+pep440-post-branch: 1.0.post1+g250b7ca
 pep440-old: 1.0.post1
 git-describe: 1.0-1-g250b7ca
 git-describe-long: 1.0-1-g250b7ca
@@ -165,9 +232,12 @@ git-describe-long: 1.0-1-g250b7ca
 closest-tag: 1.0
 distance: 1
 dirty: True
+branch: feature
 pep440: 1.0+1.g250b7ca.dirty
-pep440-pre: 1.0.post.dev1
+pep440-branch: 1.0.dev0+1.g250b7ca.dirty
+pep440-pre: 1.0.post0.dev1
 pep440-post: 1.0.post1.dev0+g250b7ca
+pep440-post-branch: 1.0.post1.dev0+g250b7ca.dirty
 pep440-old: 1.0.post1.dev0
 git-describe: 1.0-1-g250b7ca-dirty
 git-describe-long: 1.0-1-g250b7ca-dirty
@@ -176,9 +246,12 @@ git-describe-long: 1.0-1-g250b7ca-dirty
 closest-tag: 1.0+plus
 distance: 1
 dirty: False
+branch: feature
 pep440: 1.0+plus.1.g250b7ca
-pep440-pre: 1.0+plus.post.dev1
+pep440-branch: 1.0+plus.dev0.1.g250b7ca
+pep440-pre: 1.0+plus.post0.dev1
 pep440-post: 1.0+plus.post1.g250b7ca
+pep440-post-branch: 1.0+plus.post1.dev0.g250b7ca
 pep440-old: 1.0+plus.post1
 git-describe: 1.0+plus-1-g250b7ca
 git-describe-long: 1.0+plus-1-g250b7ca
@@ -186,9 +259,12 @@ git-describe-long: 1.0+plus-1-g250b7ca
 closest-tag: 1.0+plus
 distance: 1
 dirty: True
+branch: master
 pep440: 1.0+plus.1.g250b7ca.dirty
-pep440-pre: 1.0+plus.post.dev1
+pep440-branch: 1.0+plus.1.g250b7ca.dirty
+pep440-pre: 1.0+plus.post0.dev1
 pep440-post: 1.0+plus.post1.dev0.g250b7ca
+pep440-post-branch: 1.0+plus.post1.g250b7ca.dirty
 pep440-old: 1.0+plus.post1.dev0
 git-describe: 1.0+plus-1-g250b7ca-dirty
 git-describe-long: 1.0+plus-1-g250b7ca-dirty
@@ -197,9 +273,12 @@ git-describe-long: 1.0+plus-1-g250b7ca-dirty
 closest-tag: None
 distance: 1
 dirty: False
+branch: master
 pep440: 0+untagged.1.g250b7ca
-pep440-pre: 0.post.dev1
+pep440-branch: 0+untagged.1.g250b7ca
+pep440-pre: 0.post0.dev1
 pep440-post: 0.post1+g250b7ca
+pep440-post-branch: 0.post1+g250b7ca
 pep440-old: 0.post1
 git-describe: 250b7ca
 git-describe-long: 250b7ca
@@ -207,9 +286,12 @@ git-describe-long: 250b7ca
 closest-tag: None
 distance: 1
 dirty: True
+branch: feature
 pep440: 0+untagged.1.g250b7ca.dirty
-pep440-pre: 0.post.dev1
+pep440-branch: 0.dev0+untagged.1.g250b7ca.dirty
+pep440-pre: 0.post0.dev1
 pep440-post: 0.post1.dev0+g250b7ca
+pep440-post-branch: 0.post1.dev0+g250b7ca.dirty
 pep440-old: 0.post1.dev0
 git-describe: 250b7ca-dirty
 git-describe-long: 250b7ca-dirty
@@ -219,8 +301,9 @@ git-describe-long: 250b7ca-dirty
 class RenderPieces(unittest.TestCase):
     def do_render(self, pieces):
         out = {}
-        for style in ["pep440", "pep440-pre", "pep440-post", "pep440-old",
-                      "git-describe", "git-describe-long"]:
+        for style in ["pep440", "pep440-branch", "pep440-pre", "pep440-post",
+                      "pep440-post-branch", "pep440-old", "git-describe",
+                      "git-describe-long"]:
             out[style] = render(pieces, style)["version"]
         DEFAULT = "pep440"
         self.assertEqual(render(pieces, ""), render(pieces, DEFAULT))
@@ -254,6 +337,8 @@ class RenderPieces(unittest.TestCase):
                 more_pieces["closest-tag"] = value
                 if value == "None":
                     more_pieces["closest-tag"] = None
+            elif name == "branch":
+                more_pieces["branch"] = value
             else:
                 expected[name] = value
         if more_pieces and expected:
@@ -380,20 +465,18 @@ class Repo(common.Common, unittest.TestCase):
         out = set(remove_pyc(self.git("status", "--porcelain").splitlines()))
         def pf(fn):
             return os.path.normpath(os.path.join(self.project_sub_dir, fn))
-        expected = set(["A  %s" % pf(".gitattributes"),
-                        "M  %s" % pf("MANIFEST.in"),
-                        "A  %s" % pf("src/demo/_version.py"),
-                        ])
+        expected = {"A  %s" % pf(".gitattributes"),
+                    "M  %s" % pf("MANIFEST.in"),
+                    "A  %s" % pf("src/demo/_version.py"),
+                    }
         if not script_only:
             expected.add("M  %s" % pf("src/demo/__init__.py"))
         self.assertEqual(out, expected)
         if not script_only:
-            f = open(self.project_file("src/demo/__init__.py"))
-            i = f.read().splitlines()
-            f.close()
-            self.assertEqual(i[-3], "from ._version import get_versions")
-            self.assertEqual(i[-2], "__version__ = get_versions()['version']")
-            self.assertEqual(i[-1], "del get_versions")
+            with open(self.project_file("src/demo/__init__.py")) as fobj:
+                i = fobj.read().splitlines()
+            self.assertEqual(i[-2], "from . import _version")
+            self.assertEqual(i[-1], "__version__ = _version.get_versions()['version']")
         self.git("commit", "-m", "add _version stuff")
 
         # "versioneer.py setup" should be idempotent
@@ -406,7 +489,7 @@ class Repo(common.Common, unittest.TestCase):
         self.assertEqual(out[2], " 'versioneer.py' already in MANIFEST.in")
         self.assertEqual(out[3], " versionfile_source already in MANIFEST.in")
         out = set(remove_pyc(self.git("status", "--porcelain").splitlines()))
-        self.assertEqual(out, set([]))
+        self.assertEqual(out, set())
 
         UNABLE = "unable to compute version"
         NOTAG = "no suitable tags"
@@ -428,9 +511,8 @@ class Repo(common.Common, unittest.TestCase):
         # necessarily clean.
 
         # S2: dirty the pre-tagged tree
-        f = open(self.project_file("setup.py"), "a")
-        f.write("# dirty\n")
-        f.close()
+        with open(self.project_file("setup.py"), "a") as fobj:
+            fobj.write("# dirty\n")
         full = self.git("rev-parse", "HEAD")
         short = "0+untagged.2.g%s.dirty" % full[:7]
         self.do_checks("S2", {"TA": [short, full, True, None],
@@ -460,9 +542,8 @@ class Repo(common.Common, unittest.TestCase):
                               })
 
         # S4: now we dirty the tree
-        f = open(self.project_file("setup.py"), "a")
-        f.write("# dirty\n")
-        f.close()
+        with open(self.project_file("setup.py"), "a") as fobj:
+            fobj.write("# dirty\n")
         full = self.git("rev-parse", "HEAD")
         short = "1.0+0.g%s.dirty" % full[:7]
         self.do_checks("S4", {"TA": [short, full, True, None],
@@ -485,9 +566,8 @@ class Repo(common.Common, unittest.TestCase):
                               })
 
         # S6: dirty the post-tag tree
-        f = open(self.project_file("setup.py"), "a")
-        f.write("# more dirty\n")
-        f.close()
+        with open(self.project_file("setup.py"), "a") as fobj:
+            fobj.write("# more dirty\n")
         full = self.git("rev-parse", "HEAD")
         short = "1.0+1.g%s.dirty" % full[:7]
         self.do_checks("S6", {"TA": [short, full, True, None],
@@ -523,9 +603,8 @@ class Repo(common.Common, unittest.TestCase):
         self.git("archive", "--format=tar", "--prefix=demoapp-TD/",
                  "--output=../demo.tar", "HEAD")
         os.mkdir(self.subpath("out/TD"))
-        t = tarfile.TarFile(self.subpath("demo.tar"))
-        t.extractall(path=self.subpath("out/TD"))
-        t.close()
+        with tarfile.TarFile(self.subpath("demo.tar")) as t:
+            t.extractall(path=self.subpath("out/TD"))
         self.check_version(os.path.join(target, self.project_sub_dir),
                            state, "TD", exps["TD"])
 
@@ -540,9 +619,8 @@ class Repo(common.Common, unittest.TestCase):
         self.assertEqual(distfile, "demo-%s.tar" % exps["TE"][0])
         fn = os.path.join(dist_path, distfile)
         os.mkdir(self.subpath("out/TE"))
-        t = tarfile.TarFile(fn)
-        t.extractall(path=self.subpath("out/TE"))
-        t.close()
+        with tarfile.TarFile(fn) as t:
+            t.extractall(path=self.subpath("out/TE"))
         target = self.subpath("out/TE/demo-%s" % exps["TE"][0])
         self.assertTrue(os.path.isdir(target))
         self.check_version(target, state, "TE", exps["TE"])
@@ -573,7 +651,7 @@ class Repo(common.Common, unittest.TestCase):
                     "--build-scripts=build/lib", workdir=workdir)
         build_lib = os.path.join(workdir, "build", "lib")
         out = self.python("rundemo", "--version", workdir=build_lib)
-        data = dict([line.split(":",1) for line in out.splitlines()])
+        data = dict(line.split(":",1) for line in out.splitlines())
         self.compare(data["__version__"], exp_version, state, tree, "RB")
         self.assertPEP440(data["__version__"], state, tree, "RB")
         self.compare(data["version"], exp_version, state, tree, "RB")
@@ -590,7 +668,8 @@ class Repo(common.Common, unittest.TestCase):
     def assertPEP440(self, got, state, tree, runtime):
         where = "/".join([state, tree, runtime])
         pv = parse_version(got)
-        self.assertFalse(isinstance(pv, SetuptoolsLegacyVersion),
+        # rather than using an undocumented API, setuptools dev recommends this
+        self.assertFalse("Legacy" in pv.__class__.__name__,
                          "%s: '%s' was not pep440-compatible"
                          % (where, got))
         self.assertEqual(str(pv), got,
