@@ -1,31 +1,24 @@
-from __future__ import print_function
-
 import os, sys, shutil, unittest, tempfile, tarfile, virtualenv, warnings
+from wheel.bdist_wheel import get_abi_tag, get_platform
+from packaging.tags import interpreter_name, interpreter_version
 
 sys.path.insert(0, "src")
 from from_file import versions_from_file
 import common
 
+
 pyver_major = "py%d" % sys.version_info[0]
 pyver = "py%d.%d" % sys.version_info[:2]
 
-if not hasattr(unittest, "skip"): # py26
-    def _skip(reason):
-        def _skip_decorator(f):
-            def _skipped_test(*args, **kwargs):
-                print("skipping: %s" % reason)
-            return _skipped_test
-        return _skip_decorator
-    unittest.skip = _skip
 
-if not hasattr(unittest, "expectedFailure"):
-    def _expectedFailure(reason="expected to fail"):
-        def _ef_decorator(f):
-            def _ef(*args, **kwargs):
-                print("skipping: %s" % reason)
-            return _ef
-        return _ef_decorator
-    unittest.expectedFailure = _expectedFailure
+# For binary wheels with native code
+impl, impl_ver = interpreter_name(), interpreter_version()
+abi = get_abi_tag()
+try:
+    plat = get_platform(None)
+except TypeError:  # wheel < 0.34
+    plat = get_platform()
+
 
 class _Invocations(common.Common):
     def setUp(self):
@@ -65,28 +58,23 @@ class _Invocations(common.Common):
         with warnings.catch_warnings():
             warnings.simplefilter("ignore")
             virtualenv.create_environment(venv_dir)
+            self.run_in_venv(venv_dir, venv_dir,
+                             'pip', 'install', '-U',
+                             'pip', 'wheel', 'packaging')
         return venv_dir
 
     def run_in_venv(self, venv, workdir, command, *args):
         bins = {"python": os.path.join(venv, "bin", "python"),
                 "pip": os.path.join(venv, "bin", "pip"),
                 "rundemo": os.path.join(venv, "bin", "rundemo"),
-                "easy_install": os.path.join(venv, "bin", "easy_install"),
                 }
         if command == "pip":
-            args = ["--no-cache-dir"] + list(args)
-            maj, min = sys.version_info[0:2]
-            if ((maj == 2 and min >= 7) or
-                (maj == 3 and min >= 4) or
-                maj > 3):
-                # We prefer pip --isolated, but py2.6/py3.2/py3.3 (at least
-                # on travis) can't handle the --no-user-cfg that it uses
-                args = ["--isolated"] + list(args)
+            args = ["--isolated", "--no-cache-dir"] + list(args)
         return self.command(bins[command], *args, workdir=workdir)
 
     def check_in_venv(self, venv):
         out = self.run_in_venv(venv, venv, "rundemo")
-        v = dict([line.split(":", 1) for line in out.splitlines()])
+        v = dict(line.split(":", 1) for line in out.splitlines())
         self.assertEqual(v["version"], "2.0")
         return v
 
@@ -119,7 +107,7 @@ class _Invocations(common.Common):
         self.git("init", workdir=libdir)
         self.python("versioneer.py", "setup", workdir=libdir)
         self.git("add", "--all", workdir=libdir)
-        self.git("commit", "-m", "commemt", workdir=libdir)
+        self.git("commit", "-m", "comment", workdir=libdir)
         self.git("tag", "demolib-1.0", workdir=libdir)
         self.python("setup.py", "sdist", "--format=tar", workdir=libdir)
         created = os.path.join(libdir, "dist", "demolib-1.0.tar")
@@ -154,7 +142,7 @@ class _Invocations(common.Common):
         self.git("init", workdir=repodir)
         self.python("versioneer.py", "setup", workdir=repodir)
         self.git("add", "--all", workdir=repodir)
-        self.git("commit", "-m", "commemt", workdir=repodir)
+        self.git("commit", "-m", "comment", workdir=repodir)
         self.git("tag", "demoapp2-2.0", workdir=repodir)
         return repodir
 
@@ -169,7 +157,7 @@ class _Invocations(common.Common):
         self.git("init", workdir=repodir)
         self.python("versioneer.py", "setup", workdir=projectdir)
         self.git("add", "--all", workdir=repodir)
-        self.git("commit", "-m", "commemt", workdir=repodir)
+        self.git("commit", "-m", "comment", workdir=repodir)
         self.git("tag", "demoapp2-2.0", workdir=repodir)
         return projectdir
 
@@ -223,9 +211,8 @@ class _Invocations(common.Common):
         if os.path.exists(unpack_into):
             shutil.rmtree(unpack_into)
         os.mkdir(unpack_into)
-        t = tarfile.TarFile(sdist)
-        t.extractall(path=unpack_into)
-        t.close()
+        with tarfile.TarFile(sdist) as t:
+            t.extractall(path=unpack_into)
         unpacked = os.path.join(unpack_into, "demoapp2-2.0")
         self.assertTrue(os.path.exists(unpacked))
         return unpacked
@@ -236,9 +223,8 @@ class _Invocations(common.Common):
         if os.path.exists(unpack_into):
             shutil.rmtree(unpack_into)
         os.mkdir(unpack_into)
-        t = tarfile.TarFile(sdist)
-        t.extractall(path=unpack_into)
-        t.close()
+        with tarfile.TarFile(sdist) as t:
+            t.extractall(path=unpack_into)
         unpacked = os.path.join(unpack_into, "demoapp2-2.0")
         self.assertTrue(os.path.exists(unpacked))
         return unpacked
@@ -253,10 +239,50 @@ class _Invocations(common.Common):
         self.git("init", workdir=repodir)
         self.python("versioneer.py", "setup", workdir=repodir)
         self.git("add", "--all", workdir=repodir)
-        self.git("commit", "-m", "commemt", workdir=repodir)
+        self.git("commit", "-m", "comment", workdir=repodir)
         self.git("tag", "demoapp2-2.0", workdir=repodir)
         return repodir
 
+    def make_setuptools_extension_unpacked(self):
+        sdist = self.make_setuptools_extension_sdist()
+        unpack_into = self.subpath("demoappext-setuptools-unpacked")
+        if os.path.exists(unpack_into):
+            shutil.rmtree(unpack_into)
+        os.mkdir(unpack_into)
+        with tarfile.TarFile(sdist) as t:
+            t.extractall(path=unpack_into)
+        unpacked = os.path.join(unpack_into, "demoappext-2.0")
+        self.assertTrue(os.path.exists(unpacked))
+        return unpacked
+
+    def make_setuptools_extension_sdist(self):
+        # create an sdist tarball of demoappext-setuptools at 2.0
+        demoappext_setuptools_sdist = self.subpath("cache", "setuptools",
+                                                   "demoappext-2.0.tar")
+        if os.path.exists(demoappext_setuptools_sdist):
+            return demoappext_setuptools_sdist
+        repodir = self.make_setuptools_extension_repo()
+        self.python("setup.py", "sdist", "--format=tar", workdir=repodir)
+        created = os.path.join(repodir, "dist", "demoappext-2.0.tar")
+        self.assertTrue(os.path.exists(created), created)
+        shutil.copyfile(created, demoappext_setuptools_sdist)
+        return demoappext_setuptools_sdist
+
+    def make_setuptools_extension_repo(self):
+        # create a clean repo of demoappext-setuptools at 2.0
+        repodir = self.subpath("demoappext-setuptools-repo")
+        if os.path.exists(repodir):
+            shutil.rmtree(repodir)
+        # import ipdb; ipdb.set_trace()
+        shutil.copytree("test/demoappext-setuptools", repodir)
+        shutil.copy("versioneer.py", repodir)
+        self.git("init", workdir=repodir)
+        self.python("versioneer.py", "setup", workdir=repodir)
+        self.git("add", "--all", workdir=repodir)
+        self.git("commit", "-m", "comment", workdir=repodir)
+        self.git("tag", "demoappext-2.0", workdir=repodir)
+        return repodir
+
     def make_setuptools_repo_subproject(self):
         # create a clean repo of demoapp2-setuptools at 2.0
         repodir = self.subpath("demoapp2-setuptools-repo-subproject")
@@ -268,7 +294,7 @@ class _Invocations(common.Common):
         self.git("init", workdir=repodir)
         self.python("versioneer.py", "setup", workdir=projectdir)
         self.git("add", "--all", workdir=repodir)
-        self.git("commit", "-m", "commemt", workdir=repodir)
+        self.git("commit", "-m", "comment", workdir=repodir)
         self.git("tag", "demoapp2-2.0", workdir=repodir)
         return projectdir
 
@@ -303,9 +329,8 @@ class _Invocations(common.Common):
         if os.path.exists(unpack_into):
             shutil.rmtree(unpack_into)
         os.mkdir(unpack_into)
-        t = tarfile.TarFile(sdist)
-        t.extractall(path=unpack_into)
-        t.close()
+        with tarfile.TarFile(sdist) as t:
+            t.extractall(path=unpack_into)
         unpacked = os.path.join(unpack_into, "demoapp2-2.0")
         self.assertTrue(os.path.exists(unpacked))
         return unpacked
@@ -316,9 +341,8 @@ class _Invocations(common.Common):
         if os.path.exists(unpack_into):
             shutil.rmtree(unpack_into)
         os.mkdir(unpack_into)
-        t = tarfile.TarFile(sdist)
-        t.extractall(path=unpack_into)
-        t.close()
+        with tarfile.TarFile(sdist) as t:
+            t.extractall(path=unpack_into)
         unpacked = os.path.join(unpack_into, "demoapp2-2.0")
         self.assertTrue(os.path.exists(unpacked))
         return unpacked
@@ -373,6 +397,10 @@ class _Invocations(common.Common):
         shutil.copyfile(created, demoapp2_setuptools_wheel)
         return demoapp2_setuptools_wheel
 
+    def make_binary_wheelname(self, app):
+        return "%s-2.0-%s-%s-%s.whl" % (app,
+            "".join([impl, impl_ver]), abi, plat.replace("-", "_"))
+
 
 class DistutilsRepo(_Invocations, unittest.TestCase):
     def test_build(self):
@@ -403,19 +431,17 @@ class DistutilsRepo(_Invocations, unittest.TestCase):
 
     def test_sdist(self):
         sdist = self.make_distutils_sdist() # asserts version as a side-effect
-        t = tarfile.TarFile(sdist)
         # make sure we used distutils/sdist, not setuptools/sdist
-        self.assertFalse("demoapp2-2.0/src/demoapp2.egg-info/PKG-INFO" in
-                         t.getnames())
-        t.close()
+        with tarfile.TarFile(sdist) as t:
+            self.assertFalse("demoapp2-2.0/src/demoapp2.egg-info/PKG-INFO" in
+                             t.getnames())
 
     def test_sdist_subproject(self):
         sdist = self.make_distutils_sdist_subproject()
-        t = tarfile.TarFile(sdist)
         # make sure we used distutils/sdist, not setuptools/sdist
-        self.assertFalse("demoapp2-2.0/src/demoapp2.egg-info/PKG-INFO" in
-                         t.getnames())
-        t.close()
+        with tarfile.TarFile(sdist) as t:
+            self.assertFalse("demoapp2-2.0/src/demoapp2.egg-info/PKG-INFO" in
+                             t.getnames())
 
     def test_pip_install(self):
         repodir = self.make_distutils_repo()
@@ -423,7 +449,6 @@ class DistutilsRepo(_Invocations, unittest.TestCase):
         self.run_in_venv(venv, repodir, "pip", "install", ".")
         self.check_in_venv(venv)
 
-    @unittest.expectedFailure
     def test_pip_install_subproject(self):
         projectdir = self.make_distutils_repo_subproject()
         venv = self.make_venv("distutils-repo-pip-install-subproject")
@@ -436,7 +461,6 @@ class DistutilsRepo(_Invocations, unittest.TestCase):
         self.run_in_venv(venv, venv, "pip", "install", repodir)
         self.check_in_venv(venv)
 
-    @unittest.expectedFailure
     def test_pip_install_from_afar_subproject(self):
         projectdir = self.make_distutils_repo_subproject()
         venv = self.make_venv("distutils-repo-pip-install-from-afar-subproject")
@@ -476,29 +500,6 @@ class SetuptoolsRepo(_Invocations, unittest.TestCase):
         self.run_in_venv(venv, projectdir, "python", "setup.py", "install")
         self.check_in_venv_withlib(venv)
 
-    @unittest.skip("setuptools 'easy_install .': known to be broken")
-    def test_easy_install(self):
-        # This case still fails: the 'easy_install' command modifies the
-        # repo's setup.cfg (copying our --index-url and --find-links
-        # arguments into [easy_install]index_url= settings, so that any
-        # dependencies setup_requires= builds will use them), which means the
-        # repo is always "dirty", which creates an .egg with the wrong
-        # version. I have not yet found a clean way to hook the easy_install
-        # command to fix this: there is very little linkage between the
-        # parent command (which could calculate the version before setup.cfg
-        # is modified) and the command which builds the .egg. Leave it broken
-        # for now.
-        linkdir = self.make_linkdir()
-        indexdir = self.make_empty_indexdir()
-        repodir = self.make_setuptools_repo()
-        venv = self.make_venv("setuptools-repo-easy-install")
-        self.run_in_venv(venv, repodir,
-                         "python", "setup.py", "easy_install",
-                         "--index-url", indexdir, "--find-links", linkdir,
-                         "."
-                         )
-        self.check_in_venv_withlib(venv)
-
     def test_develop(self):
         linkdir = self.make_linkdir()
         indexdir = self.make_empty_indexdir()
@@ -538,19 +539,15 @@ class SetuptoolsRepo(_Invocations, unittest.TestCase):
 
     def test_sdist(self):
         sdist = self.make_setuptools_sdist() # asserts version as a side-effect
-        t = tarfile.TarFile(sdist)
         # make sure we used setuptools/sdist, not distutils/sdist
-        self.assertTrue("demoapp2-2.0/src/demoapp2.egg-info/PKG-INFO" in
-                        t.getnames())
-        t.close()
+        with tarfile.TarFile(sdist) as t:
+            self.assertIn("demoapp2-2.0/src/demoapp2.egg-info/PKG-INFO", t.getnames())
 
     def test_sdist_subproject(self):
         sdist = self.make_setuptools_sdist_subproject()
-        t = tarfile.TarFile(sdist)
         # make sure we used setuptools/sdist, not distutils/sdist
-        self.assertTrue("demoapp2-2.0/src/demoapp2.egg-info/PKG-INFO" in
-                        t.getnames())
-        t.close()
+        with tarfile.TarFile(sdist) as t:
+            self.assertIn("demoapp2-2.0/src/demoapp2.egg-info/PKG-INFO", t.getnames())
 
     def test_pip_install(self):
         linkdir = self.make_linkdir()
@@ -560,7 +557,6 @@ class SetuptoolsRepo(_Invocations, unittest.TestCase):
                          "--no-index", "--find-links", linkdir)
         self.check_in_venv_withlib(venv)
 
-    @unittest.expectedFailure
     def test_pip_install_subproject(self):
         linkdir = self.make_linkdir()
         projectdir = self.make_setuptools_repo_subproject()
@@ -577,7 +573,6 @@ class SetuptoolsRepo(_Invocations, unittest.TestCase):
                          "--no-index", "--find-links", linkdir)
         self.check_in_venv_withlib(venv)
 
-    @unittest.expectedFailure
     def test_pip_install_from_afar_subproject(self):
         linkdir = self.make_linkdir()
         projectdir = self.make_setuptools_repo_subproject()
@@ -617,17 +612,6 @@ class DistutilsSdist(_Invocations, unittest.TestCase):
                          "pip", "install", sdist)
         self.check_in_venv(venv)
 
-    def test_easy_install(self):
-        linkdir = self.make_linkdir()
-        indexdir = self.make_empty_indexdir()
-        sdist = self.make_distutils_sdist()
-        venv = self.make_venv("distutils-sdist-easy-install")
-        self.run_in_venv(venv, venv,
-                         "easy_install",
-                         "--index-url", indexdir, "--find-links", linkdir,
-                         sdist)
-        self.check_in_venv(venv)
-
 class SetuptoolsSdist(_Invocations, unittest.TestCase):
     def test_pip_install(self):
         linkdir = self.make_linkdir()
@@ -649,17 +633,6 @@ class SetuptoolsSdist(_Invocations, unittest.TestCase):
                          sdist)
         self.check_in_venv_withlib(venv)
 
-    def test_easy_install(self):
-        linkdir = self.make_linkdir()
-        indexdir = self.make_empty_indexdir()
-        sdist = self.make_setuptools_sdist()
-        venv = self.make_venv("setuptools-sdist-easy-install")
-        self.run_in_venv(venv, venv,
-                         "easy_install",
-                         "--index-url", indexdir, "--find-links", linkdir,
-                         sdist)
-        self.check_in_venv_withlib(venv)
-
 class SetuptoolsWheel(_Invocations, unittest.TestCase):
     def test_pip_install(self):
         linkdir = self.make_linkdir()
@@ -671,18 +644,6 @@ class SetuptoolsWheel(_Invocations, unittest.TestCase):
                          wheel)
         self.check_in_venv_withlib(venv)
 
-class Egg(_Invocations, unittest.TestCase):
-    def test_easy_install(self):
-        linkdir = self.make_linkdir()
-        indexdir = self.make_empty_indexdir()
-        egg = self.make_setuptools_egg()
-        venv = self.make_venv("setuptools-egg-easy-install")
-        self.run_in_venv(venv, venv,
-                         "easy_install",
-                         "--index-url", indexdir, "--find-links", linkdir,
-                         egg)
-        self.check_in_venv_withlib(venv)
-
 class DistutilsUnpacked(_Invocations, unittest.TestCase):
     def test_build(self):
         unpacked = self.make_distutils_unpacked()
@@ -758,18 +719,6 @@ class SetuptoolsUnpacked(_Invocations, unittest.TestCase):
                          "python", "setup.py", "install")
         self.check_in_venv_withlib(venv)
 
-    def test_easy_install(self):
-        linkdir = self.make_linkdir()
-        indexdir = self.make_empty_indexdir()
-        unpacked = self.make_setuptools_unpacked()
-        venv = self.make_venv("setuptools-unpacked-easy-install")
-        self.run_in_venv(venv, unpacked,
-                         "python", "setup.py", "easy_install",
-                         "--index-url", indexdir, "--find-links", linkdir,
-                         "."
-                         )
-        self.check_in_venv_withlib(venv)
-
     def test_wheel(self):
         unpacked = self.make_setuptools_unpacked()
         self.python("setup.py", "bdist_wheel", workdir=unpacked)
@@ -813,6 +762,50 @@ class SetuptoolsUnpacked(_Invocations, unittest.TestCase):
                          "--no-index", "--find-links", linkdir)
         self.check_in_venv_withlib(venv)
 
+    def test_extension_wheel_setuptools(self):
+        # create an wheel of demoappext-setuptools at 2.0
+        wheelname = self.make_binary_wheelname('demoappext')
+        demoappext_setuptools_wheel = self.subpath("cache", "setuptools",
+                                                   wheelname)
+        if os.path.exists(demoappext_setuptools_wheel):
+            # there are two ways to make this .whl, and we need to exercise
+            # both, so don't actually cache the results
+            os.unlink(demoappext_setuptools_wheel)
+        repodir = self.make_setuptools_extension_repo()
+        self.python("setup.py", "bdist_wheel", workdir=repodir)
+        created = os.path.join(repodir, "dist", wheelname)
+        self.assertTrue(os.path.exists(created), created)
+
+    def test_extension_inplace(self):
+        # build extensions in place. No wheel package
+        unpacked = self.make_setuptools_extension_unpacked()
+        venv = self.make_venv("setuptools-unpacked-pip-wheel-extension")
+        self.run_in_venv(venv, unpacked,
+                         "python", "setup.py", "build_ext", "-i")
+        # No wheel package is created, _version.py should exist in
+        # module dir only
+        version_file = os.path.join(unpacked, "demo", "_version.py")
+        self.assertTrue(os.path.exists(version_file))
+
+    def test_extension_wheel_pip(self):
+        # create an wheel of demoappext-setuptools at 2.0 with pip
+        wheelname = self.make_binary_wheelname('demoappext')
+        demoappext_setuptools_wheel = self.subpath("cache", "setuptools",
+                                                   wheelname)
+        if os.path.exists(demoappext_setuptools_wheel):
+            # there are two ways to make this .whl, and we need to exercise
+            # both, so don't actually cache the results
+            os.unlink(demoappext_setuptools_wheel)
+        unpacked = self.make_setuptools_extension_unpacked()
+        linkdir = self.make_linkdir()
+        venv = self.make_venv("setuptools-unpacked-pip-wheel-extension")
+        self.run_in_venv(venv, unpacked,
+                         "pip", "wheel", "--wheel-dir", "wheelhouse",
+                         "--no-index", "--find-links", linkdir,
+                         ".")
+        created = os.path.join(unpacked, "wheelhouse", wheelname)
+        self.assertTrue(os.path.exists(created), created)
+
 
 if __name__ == '__main__':
     unittest.main()
